==> ui-framework-4.5.4-sources.jar: io/intino/alexandria/ui/displays/AbstractRouteDispatcher.java <==
package io.intino.alexandria.ui.displays;

import io.intino.alexandria.ui.Soul;

import java.util.ArrayList;
import java.util.Collections;
import java.util.HashMap;
import java.util.List;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

public abstract class AbstractRouteDispatcher implements io.intino.alexandria.ui.displays.DisplayRouteDispatcher {
    private static java.util.Map<String, String> patterns = new HashMap<>();

    public AbstractRouteDispatcher() {
        registerPatterns();
    }

    @Override
    public void dispatch(Soul soul, String address) {
        address = address.replaceFirst(soul.session().browser().basePath(), "");
        List<String> params = paramsOf(address);
        if (address.length() <= 1) { dispatchHome(soul); return; }
        if (address.matches(patterns.get("home"))) { dispatchHome(soul); return; }
        if (address.matches(patterns.get("docs"))) { dispatchDocs(soul); return; }
        if (address.matches(patterns.get("widgetType"))) { dispatchWidgetType(soul, params.get(0)); return; }
    }

    public abstract void dispatchHome(Soul soul);
    public abstract void dispatchDocs(Soul soul);
    public abstract void dispatchWidgetType(Soul soul, String type);

    private void registerPatterns() {
        if (patterns.size() > 0) return;
        patterns.put("home", "");
        patterns.put("docs", "\\/docs");
        patterns.put("widgetType", "\\/docs\\/data\\/([^\\/]*)");
    }

    private String patternOf(String address) {
        if (address.matches(patterns.get("home"))) return patterns.get("home");
        else if (address.matches(patterns.get("home"))) return patterns.get("home");
        else if (address.matches(patterns.get("docs"))) return patterns.get("docs");
        else if (address.matches(patterns.get("widgetType"))) return patterns.get("widgetType");
        return null;
    }

    private List<String> paramsOf(String address) {
        return paramsOf(address, patternOf(address));
    }

    private List<String> paramsOf(String address, String pattern) {
        if (pattern == null) return java.util.Collections.emptyList();
        Pattern p = Pattern.compile(pattern);
        Matcher m = p.matcher(address);
        if (!m.find()) return Collections.emptyList();
        List<String> result = new ArrayList<>();
        for (int i=1; i<=m.groupCount(); i++) result.add(m.group(i));
        return result;
    }

}

==> ui-framework-4.13.4-sources.jar: io/intino/alexandria/ui/displays/AbstractRouteDispatcher.java <==
package io.intino.alexandria.ui.displays;

import io.intino.alexandria.ui.Soul;

import java.util.ArrayList;
import java.util.Collections;
import java.util.HashMap;
import java.util.List;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

public abstract class AbstractRouteDispatcher implements io.intino.alexandria.ui.displays.DisplayRouteDispatcher {
    private static java.util.Map<String, String> patterns = new HashMap<>();

    public AbstractRouteDispatcher() {
        registerPatterns();
    }

    @Override
    public void dispatch(Soul soul, String address) {
        address = address.replaceFirst(soul.session().browser().basePath(), "");
        List<String> params = paramsOf(address);
        if (address.length() <= 1) { dispatchHome(soul); return; }
        if (address.matches(patterns.get("home"))) { dispatchHome(soul); return; }
        if (address.matches(patterns.get("docs"))) { dispatchDocs(soul); return; }
        if (address.matches(patterns.get("widgetType"))) { dispatchWidgetType(soul, params.get(0)); return; }
        if (address.matches(patterns.get("microSite"))) { dispatchMicroSite(soul, params.get(0)); return; }
    }

    public abstract void dispatchHome(Soul soul);
    public abstract void dispatchDocs(Soul soul);
    public abstract void dispatchWidgetType(Soul soul, String type);
    public abstract void dispatchMicroSite(Soul soul, String page);

    private void registerPatterns() {
        if (patterns.size() > 0) return;
        patterns.put("home", "");
        patterns.put("docs", "\\/docs");
        patterns.put("widgetType", "\\/docs\\/data\\/([^\\/]*)");
        patterns.put("microSite", "\\/docs\\/data\\/microsite\\/([^\\/]*)");
    }

    private String patternOf(String address) {
        if (address.matches(patterns.get("home"))) return patterns.get("home");
        else if (address.matches(patterns.get("home"))) return patterns.get("home");
        else if (address.matches(patterns.get("docs"))) return patterns.get("docs");
        else if (address.matches(patterns.get("widgetType"))) return patterns.get("widgetType");
        else if (address.matches(patterns.get("microSite"))) return patterns.get("microSite");
        return null;
    }

    private List<String> paramsOf(String address) {
        return paramsOf(address, patternOf(address));
    }

    private List<String> paramsOf(String address, String pattern) {
        if (pattern == null) return java.util.Collections.emptyList();
        Pattern p = Pattern.compile(pattern);
        Matcher m = p.matcher(address);
        if (!m.find()) return Collections.emptyList();
        List<String> result = new ArrayList<>();
        for (int i=1; i<=m.groupCount(); i++) result.add(m.group(i));
        return result;
    }

}
